from PySide2 import QtWidgets, QtCore, QtGui
from ngSkinTools2.api import PaintTool
from ngSkinTools2.api.paint import popups
from ngSkinTools2.ui import qt, widgets
from ngSkinTools2.ui.session import session


def brush_settings_popup(paint):
    """

    :type paint: PaintTool
    """
    window = QtWidgets.QWidget(qt.mainWindow)
    window.setWindowFlags(QtCore.Qt.Popup | QtCore.Qt.FramelessWindowHint)
    window.setAttribute(QtCore.Qt.WA_DeleteOnClose)

    spacing = 5

    layout = QtWidgets.QVBoxLayout()
    layout.setSpacing(spacing)

    intensity_slider = widgets.NumberSliderGroup()
    widgets.set_paint_expo(intensity_slider, paint.paint_mode)
    intensity_slider.set_value(paint.brush_intensity)

    @qt.on(intensity_slider.slider.sliderReleased, intensity_slider.spinner.editingFinished)
    def close_with_slider_intensity():
        close_with_intensity(intensity_slider.value())

    def close_with_intensity(value):
        paint.brush_intensity = value
        session.events.tool_settings_changed.emit()
        window.close()

    def create_intensity_button(intensity):
        btn = QtWidgets.QPushButton("{0:.2f}".format(intensity))
        btn.clicked.connect(lambda: close_with_intensity(intensity))
        btn.setMinimumWidth(60)
        btn.setMinimumHeight(30)
        return btn

    layout.addLayout(intensity_slider.layout())

    row = QtWidgets.QHBoxLayout()
    row.setSpacing(spacing)
    row.addWidget(create_intensity_button(0.0))
    row.addWidget(create_intensity_button(1.0))
    layout.addLayout(row)

    row = QtWidgets.QHBoxLayout()
    row.setSpacing(spacing)
    row.addWidget(create_intensity_button(0.25))
    row.addWidget(create_intensity_button(0.5))
    row.addWidget(create_intensity_button(0.75))
    layout.addLayout(row)

    group = QtWidgets.QGroupBox("Brush Intensity")
    group.setLayout(layout)

    layout = QtWidgets.QVBoxLayout()
    layout.setMargin(4)
    layout.addWidget(group)

    window.setLayout(layout)

    window.show()
    mp = QtGui.QCursor.pos()
    window.move(mp.x() - window.size().width() / 2, mp.y() - window.size().height() / 2)

    window.activateWindow()

    popups.close_all()
    popups.add(window)
